import streamlit as st


def agixt_docs():
    st.markdown(
        "[![Documentation](https://img.shields.io/badge/Documentation-AGiXT-blue?logo=github&style=plastic)](https://josh-xt.github.io/AGiXT/) [![GitHub](https://img.shields.io/badge/GitHub-AGiXT-blue?logo=github&style=plastic)](https://github.com/Josh-XT/AGiXT) [![Discord](https://img.shields.io/discord/1097720481970397356?label=Discord&logo=discord&logoColor=white&style=plastic&color=5865f2)](https://discord.gg/d3TkHRZcjD) [![Twitter](https://img.shields.io/badge/Twitter-Follow_@Josh__XT-blue?logo=twitter&style=plastic)](https://twitter.com/Josh_XT)"
    )
